//
// This program shows how to use the TBarcode Library.
//

#ifdef WIN32
  #include "tbarcode.h"
#else // LINUX or UNIX
  #include <libtbarcode7/tbarcode.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define BUFFER_SIZE   0xffff
#define PATH_TBARCODE "/usr/local/share/tbarcode2"


int main(int argc, char* argv[])
{
  ERRCODE eCode;                // return value of TBarCode functions
  RECT rect;                    // bounding rectangle of the barcode
  t_BarCode* pBarCode = NULL;   // barcode handle
  void* pBuffer;                // a simple buffer
  const char* demo = "Demo12345";     // data, which will be encoded in barcode
  
  // allocate buffer
  pBuffer = malloc (BUFFER_SIZE);
  if(pBuffer==NULL)
    exit(EXIT_FAILURE);

  //
  // ------ Initialize TBarCode Library
  //

#if !defined(WIN32)
  // Initialize library (required in LINUX/UNIX)
  eCode = BCInitLibrary(PATH_TBARCODE);
  printf ("BCInitLibrary - eCode = %ld\n", eCode);
#endif
	
  // Set License Key (optional)
	eCode = BCLicenseMe("Demo-User", 
                      eLicKindSite,
                      1, 
                      "00000000000000000000000000000000",
                      eLicProd2D);

  //
  // ------ Set BarCode parameters
  //

  // Allocate barcode structure
  eCode = BCAlloc(&pBarCode);
  printf ("BCAlloc - eCode = %ld\n", eCode);

  // Set content of barcode
  eCode = BCSetText(pBarCode, demo, (LONG)strlen(demo));
  printf ("BCSetText - eCode = %ld\n", eCode);

  // Set barcode type to 'Code 128'
  eCode = BCSetBCType(pBarCode, eBC_Code128);
  printf ("BCSetBCType - eCode = %ld\n", eCode);

  // Set size of font
  eCode = BCSetFontHeight(pBarCode, 10);
  printf ("BCSetFontHeight - eCode = %ld\n", eCode);

  // Check barcode parameters
  eCode = BCCheck(pBarCode);
  printf ("BCCheck - eCode = %ld\n", eCode);
	
  // Calculate check digit
  eCode = BCCalcCD(pBarCode);
  printf ("BCCalcCD - eCode = %ld\n", eCode);

  // Create barcode
  eCode = BCCreate(pBarCode);
  printf("BCCreate - eCode = %ld\n", eCode);

  //
  // ------ Create Postscript BarCode
  //

  // Set size of barcode [1/1000mm]
  // (In Postscript the origin is the bottom, left corner.)
  rect.left = 0;       // = 0 mm
  rect.bottom = 0;     // = 0 mm
  rect.right = 50000;  // = 50 mm
  rect.top = 30000;    // = 30 mm
  
  // Save barcode as file
  eCode = BCPostscriptToFile(pBarCode, "barcode.ps", &rect);
  printf("BCPostscriptToFile - eCode = %ld\n", eCode);

  // Write barcode into a buffer
  eCode = BCPostscriptToMemory(pBarCode, pBuffer, BUFFER_SIZE, &rect);
  printf("BCPostscriptToMemory - eCode = %ld\n", eCode);

  // Show content of buffer
  printf("buffer: \n%s\n", (char *)pBuffer);

  //
  // ------ Create PCL BarCode
  //

  // Again set the size of barcode [1/1000mm]
  // (PCL uses different coordinate system than Postscript.
  // The origin is the top, left corner.)
  rect.left = 0;        // = 0 mm
  rect.top = 0;         // = 0 mm
  rect.bottom = 30000;  // = 30 mm
  rect.right = 50000;   // = 50 mm

  // Safe barcode as file
  eCode = BCPCLToFile(pBarCode, "barcode.pcl", &rect);
  printf("BCPCLToFile - eCode = %ld\n", eCode);

  // Write barcode into a buffer
  eCode = BCPCLToMemory(pBarCode, pBuffer, BUFFER_SIZE, &rect);
  printf("BCPCLToMemory - eCode = %ld\n", eCode);

  // Check if ImageMagick-Library is available, which is used to 
  // create bitmap output.
  if (BCIsImageMagickAvailable())
  {
    //
    // ------ Create BMP BarCode
    //
    // For bitmap output the origin is the top, left corner.
    rect.left = 0;
    rect.top = 0;
    rect.bottom = 30000;
    rect.right = 50000;

    // Ensure that the width of a bar is a multiple of pixels (and at minimum one pixel)
    BCSetOptResolution(pBarCode, TRUE);

    // Here we use a resolution of 96 dpi
    BCSetDPI(pBarCode, 96, 96);

    // Save barcode as BMP
    eCode = BCBitmapToFile(pBarCode, "barcode.bmp", &rect);
    printf("BCBitmapToFile(bmp) - eCode = %ld\n", eCode);

    // Save barcode as GIF
    eCode = BCBitmapToFile(pBarCode, "barcode.gif", &rect);
    printf("BCBitmapToFile(gif) - eCode = %ld\n", eCode);

    // Save barcode as JPG
    eCode = BCBitmapToFile(pBarCode, "barcode.jpg", &rect);
    printf("BCBitmapToFile(jpg) - eCode = %ld\n", eCode);

    // Save barcode as TIF
    eCode = BCBitmapToFile(pBarCode, "barcode.tif", &rect);
    printf("BCBitmapToFile(tif) - eCode = %ld\n", eCode);
  }

  //
  // ------ Clean up
  //

  // Free barcode structure
  eCode = BCFree(pBarCode);
  printf("BCFree - eCode = %ld\n", eCode);

#if !defined(WIN32)
  // Clean up (required in LINUX/UNIX)
  eCode = BCDeInitLibrary();
  printf("BCDeInitLibrary - eCode = %ld\n", eCode);
#endif

  free(pBuffer);
  
  return EXIT_SUCCESS;
}
